package ar.wss.window;

/*
 * Copyright (c) 1995, 2008, Oracle and/or its affiliates. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *   - Neither the name of Oracle or the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

import java.awt.*;
import java.awt.event.*;

import java.io.IOException;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.ServerSocket;

import java.net.UnknownHostException;
import javax.swing.*;
import javax.swing.UIManager.LookAndFeelInfo;
import javax.swing.event.*;

import ar.wss.wssServer;

/* ListDemo.java requires no other files. */
public class blimpGui extends JPanel implements ListSelectionListener {

	private static final long serialVersionUID = 8674892123449089450L;

	private static final String hireString = "start";
	private static final String stadString = "stadistic";

	private static JFrame frame;
	private static JTextField blimp_port;

	private static wssServer tHServer;

	public JButton hireButton;
	public JButton stadButton;

	public String getip() {

		try {
			InetAddress IP = InetAddress.getLocalHost();
			return IP.getHostAddress();
		} catch (UnknownHostException e) {
			e.printStackTrace();
		}
		return null;
	}

	public blimpGui() {
		super(new BorderLayout());
		JPanel mainPannel = new JPanel();
		mainPannel.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();

		hireButton = new JButton(hireString);
		HireListener hireListener = new HireListener(hireButton);
		hireButton.setActionCommand(hireString);
		hireButton.addActionListener(hireListener);
		hireButton.setEnabled(true);
		
		stadButton = new JButton(stadString);
		HireListener stadListener = new HireListener(stadButton);
		stadButton.setActionCommand(stadString);
		stadButton.addActionListener(stadListener);
		stadButton.setEnabled(false);
		
		
		blimp_port = new JTextField(10);
		JLabel lbl_port = new JLabel("Port:", SwingConstants.LEFT);
		blimp_port.setText("1235");
		c.gridx = 0;
		c.gridy = 0;
		mainPannel.add(lbl_port, c);

		c.gridx = 1;
		c.gridy = 0;
		mainPannel.add(blimp_port, c);
		
		c.gridx = 2;
		c.gridy = 0;
		mainPannel.add(hireButton, c);
		
		c.gridx = 3;
		c.gridy = 0;
		mainPannel.add(stadButton, c);


		
		mainPannel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

		add(mainPannel, BorderLayout.PAGE_END);
	}

	// This listener is shared by the text field and the hire button.
	class HireListener implements ActionListener, DocumentListener {

		private JButton button;

		public HireListener(JButton button) {
			this.button = button;
		}

		// Required by ActionListener.
		public void actionPerformed(ActionEvent e) {

			if (e.getActionCommand().equals("start")) {
				if ((available(Integer.parseInt(blimp_port.getText())))
						|| (available(Integer.parseInt(blimp_port.getText()) + 1))) {

					startBlimp(blimp_port.getText(), false);
					stadButton.setEnabled(true);
					blimp_port.setEnabled(false);
					button.setText("stop");
					button.setActionCommand("stop");
					frame.setTitle("Blimp is running");

				} else {
					JOptionPane
							.showMessageDialog(
									frame,
									"Port "
											+ blimp_port.getText()
											+ " is not available \nPlease choose a different port");
				}

			} else if (e.getActionCommand().equals("stop")) {

				tHServer.interrupt();
				tHServer = null;
				stadButton.setEnabled(false);

				button.setText("start");
				button.setActionCommand("start");
				blimp_port.setEnabled(true);

			}else if (e.getActionCommand().equals("stadistic")){
				JOptionPane
				.showMessageDialog(
						frame,
						" "
								+ tHServer.getStad());
			}
			
		}

		// Required by DocumentListener.
		public void insertUpdate(DocumentEvent e) {
			// enableButton();
		}

		// Required by DocumentListener.
		public void removeUpdate(DocumentEvent e) {
			// handleEmptyTextField(e);
		}

		// Required by DocumentListener.
		public void changedUpdate(DocumentEvent e) {
			// if (!handleEmptyTextField(e)) {
			// enableButton();
			// }
		}

	}

	// This method is required by ListSelectionListener.
	public void valueChanged(ListSelectionEvent e) {
		if (e.getValueIsAdjusting() == false) {

		}
	}

	public static boolean available(int port) {

		ServerSocket ss = null;
		DatagramSocket ds = null;
		try {
			ss = new ServerSocket(port);
			ss.setReuseAddress(true);
			ds = new DatagramSocket(port);
			ds.setReuseAddress(true);
			return true;
		} catch (IOException e) {
		} finally {
			if (ds != null) {
				ds.close();
			}

			if (ss != null) {
				try {
					ss.close();
				} catch (IOException e) {
					/* should not be thrown */
				}
			}
		}

		return false;
	}

	/**
	 * Create the GUI and show it. For thread safety, this method should be
	 * invoked from the event-dispatching thread.
	 */
	private static void createAndShowGUI() {

		// Create and set up the window.
		frame = new JFrame("Blimp");

		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		// Create and set up the content pane.
		JComponent newContentPane = new blimpGui();
		newContentPane.setOpaque(true); // content panes must be opaque
		frame.setContentPane(newContentPane);

		ImageIcon img = new ImageIcon(
				blimpGui.class.getResource("/ar/wss/files/blimp.png"));
		frame.setIconImage(img.getImage());
		// Display the window.
		frame.pack();
		frame.setVisible(true);
		frame.setResizable(false);
		Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
		frame.setLocation(dim.width / 2 - frame.getSize().width / 2, dim.height
				/ 2 - frame.getSize().height / 2);
		frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		frame.addWindowListener(new java.awt.event.WindowAdapter() {
			@Override
			public void windowClosing(java.awt.event.WindowEvent windowEvent) {

				String message = "Are you sure to close blimp?";
				String title = "close blimp?";
				// display the JOptionPane showConfirmDialog
				int reply = JOptionPane.showConfirmDialog(null, message, title,
						JOptionPane.YES_NO_OPTION);
				if (reply == JOptionPane.YES_OPTION) {
					if (tHServer != null)
						tHServer.interrupt();
					System.exit(0);
					System.out.println("by =)");
				}

			}
		});

	}

	public static void startBlimp(String port, boolean debug) {

		System.out.println("Blimp is running");

		ServerSocket Server;
		try {

			Server = new ServerSocket(Integer.parseInt(port));
			tHServer = new wssServer(Server);
			tHServer.start();

		} catch (NumberFormatException e) {
			// TODO Bloque catch generado autom�ticamente
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Bloque catch generado autom�ticamente
			e.printStackTrace();
		}

	}

	public static void main(String[] args) {
		// Schedule a job for the event-dispatching thread:
		// creating and showing this application's GUI.
		if (args.length == 2) {
			if (args[0].equals("-p")) {
				startBlimp(args[1], false);
			}
		} else if (args.length == 3) {
			if ((args[0].equals("-p")) && (args[2].equals("-d"))) {
				startBlimp(args[1], true);
			}
		} else {
			javax.swing.SwingUtilities.invokeLater(new Runnable() {

				public void run() {
					try {
						for (LookAndFeelInfo info : UIManager
								.getInstalledLookAndFeels()) {
							if ("Nimbus".equals(info.getName())) {
								UIManager.setLookAndFeel(info.getClassName());
								break;
							}
						}
					} catch (ClassNotFoundException e) {
						e.printStackTrace();
					} catch (InstantiationException e) {
						e.printStackTrace();
					} catch (IllegalAccessException e) {
						e.printStackTrace();
					} catch (UnsupportedLookAndFeelException e) {
						e.printStackTrace();
					}

					createAndShowGUI();
				}
			});
		}
	}
}
