package ar.wss;

import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintWriter;

import java.net.Socket;
import java.nio.charset.Charset;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.UUID;
import java.util.Vector;

import com.sun.xml.internal.messaging.saaj.packaging.mime.internet.MimeUtility;

import ar.wss.libs.json.JSONException;
import ar.wss.libs.json.JSONObject;

public class wssClient extends Thread {

	private static final int dlyTime = 100;
	private UUID idUUID = UUID.randomUUID();

	private String key = null;

	private boolean handshake = false;
	private boolean stop = false;

	Socket socketCl;
	private Vector<JSONObject> cacheOut = new Vector<JSONObject>();
	private Vector<JSONObject> cacheIn = new Vector<JSONObject>();

	public String getUid() {
		return idUUID.toString();
	}

	public JSONObject getMsg() {

		if (cacheOut.isEmpty())
			return null;

		JSONObject cacheOutTmp = cacheOut.get(0);
		cacheOut.remove(0);

		return cacheOutTmp;

	}

	static String encodeString(String input) throws Exception {
		MessageDigest digest = null;
		try {
			digest = MessageDigest.getInstance("SHA-1");
			byte[] inputBytes = input.getBytes();
			byte[] hashBytes = digest.digest(inputBytes);
			return new String(encode(hashBytes));
		} catch (NoSuchAlgorithmException e) {

		}
		return "";
	}

	public static byte[] encode(byte[] b) throws Exception {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		OutputStream b64os = MimeUtility.encode(baos, "base64");
		b64os.write(b);
		b64os.close();
		return baos.toByteArray();
	}

	public synchronized void add(JSONObject data) {

		cacheIn.add(data);
	}

	public void send(String msg) throws IOException {
		byte[] byMsg = brodcast(msg);
		OutputStream out = this.socketCl.getOutputStream();
		DataOutputStream dos = new DataOutputStream(out);
		dos.write(byMsg);
		dos.flush();
	}

	public void flushmsg() throws IOException {
		if (!cacheIn.isEmpty()) {
			send(((JSONObject) cacheIn.get(0)).toString());
			cacheIn.remove(0);
		}

	}

	public wssClient(Socket client) throws JSONException, IOException {

		this.socketCl = client;
		new Thread() {

			public void run() {
				while (!stop) {
					try {
						flushmsg();
						Thread.sleep(dlyTime);
					} catch (IOException | InterruptedException e) {
						// TODO Bloque catch generado autom�ticamente
						e.printStackTrace();
					}
				}
			}
		}.start();

	}

	// {UID:"",DATA:""}
	public JSONObject setMsg(String msg) {
		try {

			return new JSONObject(msg.substring(0, (msg.lastIndexOf("}") + 1)));
		} catch (JSONException e) {
			// TODO Bloque catch generado autom�ticamente
			e.printStackTrace();
		}
		return null;
	}

	// {UID:"",DATA:""}
	public String getMsg(String msg) {
		try {

			JSONObject jmsg = new JSONObject();
			jmsg.put("DATA", msg);
			// jmsg.put("UID", key);
			return jmsg.toString();
		} catch (JSONException e) {
			// TODO Bloque catch generado autom�ticamente
			e.printStackTrace();
		}

		return null;
	}

	public void sendResponse(String msg, String from) throws JSONException,
			IOException {

		JSONObject jsMsg = setMsg(msg);
		jsMsg.put("FROM", from);

		if (jsMsg != null)
			cacheOut.add(jsMsg);

	}

	public byte[] brodcast(String mess) {

		byte[] rawData = mess.getBytes(Charset.forName("UTF8"));

		int frameCount = 0;
		byte[] frame = new byte[10];

		frame[0] = (byte) 129;

		if (rawData.length <= 125) {
			frame[1] = (byte) rawData.length;
			frameCount = 2;
		} else if (rawData.length >= 126 && rawData.length <= 65535) {
			frame[1] = (byte) 126;
			int len = rawData.length;
			frame[2] = (byte) ((len >> 8) & (byte) 255);
			frame[3] = (byte) (len & (byte) 255);
			frameCount = 4;
		} else {
			frame[1] = (byte) 127;
			int len = rawData.length;
			frame[2] = (byte) ((len >> 56) & (byte) 255);
			frame[3] = (byte) ((len >> 48) & (byte) 255);
			frame[4] = (byte) ((len >> 40) & (byte) 255);
			frame[5] = (byte) ((len >> 32) & (byte) 255);
			frame[6] = (byte) ((len >> 24) & (byte) 255);
			frame[7] = (byte) ((len >> 16) & (byte) 255);
			frame[8] = (byte) ((len >> 8) & (byte) 255);
			frame[9] = (byte) (len & (byte) 255);
			frameCount = 10;
		}

		int bLength = frameCount + rawData.length;

		byte[] reply = new byte[bLength];

		int bLim = 0;
		for (int i = 0; i < frameCount; i++) {
			reply[bLim] = frame[i];
			bLim++;
		}
		for (int i = 0; i < rawData.length; i++) {
			reply[bLim] = rawData[i];
			bLim++;
		}
		return reply;
	}

	public void run() {
		try {

			while (!stop) {

				if (!handshake) {
					BufferedReader in = new BufferedReader(
							new InputStreamReader(
									this.socketCl.getInputStream()));

					String inputLine;

					while (((inputLine = in.readLine()) != null)
							&& (!(inputLine.equals("")))) {

						if (inputLine.indexOf("Sec-WebSocket-Key:") > -1) {
							key = inputLine.substring(
									"Sec-WebSocket-Key: ".length(),
									inputLine.length());
							key = key + "258EAFA5-E914-47DA-95CA-C5AB0DC85B11"; // adding

							PrintWriter out = new PrintWriter(
									this.socketCl.getOutputStream(), true);

							out.format("HTTP/1.1 101 Switching Protocolsr\n");
							out.format("Upgrade: websocket\r\n");
							out.format("Connection: Upgrade\r\n");
							out.format("Sec-WebSocket-Accept: %s\r\n",
									encodeString(key));
							out.format("\r\n");
							out.flush();
							// IN.CLOSE();

							handshake = true;
							Thread.sleep(dlyTime);

							sendResponse(
									"{\"DATA\":\"USER "
											+ this.idUUID.toString()
											+ " CONNECTED \",\"UID\":\"BLIMP_SERVER\",\"TO\":\"ALL\"}",
									"SERVER");

							//
						}

					}

				} else {

					int len;
					byte[] message;
					byte[] masks;
					byte[] b = new byte[1000];

					len = this.socketCl.getInputStream().read(b);

					if (len != -1) {

						byte rLength = 0;
						int rMaskIndex = 2;
						int rDataStart = 0;
						byte data = b[1];
						byte op = (byte) 127;
						rLength = (byte) (data & op);

						int opcode = b[0] & 0x0f; // byte 0, bits 4 - 7

						if (rLength == (byte) 126)
							rMaskIndex = 4;
						if (rLength == (byte) 127)
							rMaskIndex = 10;

						masks = new byte[4];

						int j = 0;
						int i = 0;
						for (i = rMaskIndex; i < (rMaskIndex + 4); i++) {
							masks[j] = b[i];
							j++;
						}

						rDataStart = rMaskIndex + 4;

						int messLen = len - rDataStart;

						message = new byte[messLen];

						for (i = rDataStart, j = 0; i < len; i++, j++) {
							message[j] = (byte) (b[i] ^ masks[j % 4]);
						}

						if (opcode == 8) {
							sendResponse(
									new String(
											"{\"DATA\":\"USER "
													+ this.idUUID.toString()
													+ " DISCONNECTED\",\"UID\":\"BLIMP_SERVER\",\"TO\":\"ALL\"}"),
									this.idUUID.toString());

							this.stop = true;

						} else {
							sendResponse(new String(message),
									this.idUUID.toString());
						}

						// socket.getInputStream().reset();

					} else {
						// this.socketCl.close();
					}

					masks = null;
					message = null;
					b = null;

				}
				Thread.sleep(dlyTime);
			}

		} catch (IOException e) {
			// TODO Bloque catch generado autom�ticamente
			e.printStackTrace();
		} catch (JSONException e) {
			// TODO Bloque catch generado autom�ticamente
			e.printStackTrace();
		} catch (Exception e) {
			// TODO Bloque catch generado autom�ticamente
			e.printStackTrace();
		}
	}

	public void interrupt() {
		try {
			this.socketCl.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		stop = true;

	}

}
